plot_cluster_counts <- function(sce) {

  cluster_table <- table(colLabels(sce))
  cluster_count_data <- tibble::tibble(
    cluster = names(cluster_table),
    count = as.vector(cluster_table)
  )

  ggplot(cluster_count_data, aes(x = cluster, y = count)) +
    geom_col(fill = "seagreen") +
    coord_flip() +
    labs(
      x = "Cluster",
      y = "Number of cells"
    ) +
    theme_bw()
}

plot_cluster_mean_boxplot <- function(sce) {

  cluster_means <- lapply(levels(colLabels(sce)), function(x) {
    rowMeans(logcounts(sce[, colLabels(sce) == x]))
  })
  names(cluster_means) <- levels(colLabels(sce))

  cluster_mean_data <- tibble::as_tibble(cluster_means)
  cluster_mean_data <- tidyr::pivot_longer(
    cluster_mean_data,
    everything(),
    names_to = "cluster",
    values_to = "gene_mean"
  )

  ggplot(cluster_mean_data, aes(x = cluster, y = gene_mean)) +
    geom_violin(fill = "seagreen") +
    coord_flip() +
    labs(
      x = "Cluster",
      y = "Gene mean (logcounts)"
    ) +
    theme_bw()
}

plot_median_diffs <- function(sce) {

  cluster_median_diffs <- lapply(levels(colLabels(sce)), function(x) {
    medians_1 <- rowMedians(logcounts(sce[, colLabels(sce) == x]))
    medians_2 <- rowMedians(logcounts(sce[, colLabels(sce) != x]))

    medians_1 - medians_2
  })
  names(cluster_median_diffs) <- levels(colLabels(sce))

  cluster_median_diff_data <- tibble::as_tibble(cluster_median_diffs)
  cluster_median_diff_data <- tidyr::pivot_longer(
    cluster_median_diff_data,
    everything(),
    names_to = "cluster",
    values_to = "gene_median_diff"
  )

  ggplot(cluster_median_diff_data, aes(gene_median_diff)) +
    geom_histogram(fill = "seagreen", bins = 30) +
    facet_wrap(~cluster) +
    labs(
      y = "Count",
      x = "Median difference"
    ) +
    theme_bw()
}
