# Script to extract true marker genes from the simulated datasets.

# Get the passed arguments from the command line via Snakemake.
args <- R.utils::commandArgs(trailingOnly = TRUE, asValues = TRUE)

#' Find candidate marker genes from a dataset simulated using splatter
#'
#' @param sce A SingleCellExperiment object simulated using splatter
#'
#' @details TODO
#'
find_marker_genes <- function(sce) {

  stopifnot(is(sce, "SingleCellExperiment"))

  n_groups <- length(unique(colData(sce)$Group))
  col_names <- paste0("DEFacGroup", seq_len(n_groups))
  data <- as.matrix(rowData(sce)[, col_names])

  out <- list()
  for (i in seq_len(n_groups)) {
    n_genes <- nrow(data)
    not_i <- setdiff(1:n_groups, i)

    mean_scores <- numeric(n_genes)
    max_scores <- numeric(n_genes)
    median_scores <- numeric(n_genes)
    directions <- character(n_genes)
    de_values <- numeric(n_genes)
    for (j in seq_len(n_genes)) {
      y <- data[j, not_i]
      x <- data[j, i]
      log_ratios <- log(x / y)
      mean_scores[[j]] <- mean(log_ratios)
      median_scores[[j]] <- median(log_ratios)
      max_scores[[j]] <- max(log_ratios)
      de_values[[j]] <- x
      if (x > 1) {
        directions[[j]] <- "up"
      } else if (x == 1) {
        directions[[j]] <- "background"
      } else {
        directions[[j]] <- "down"
      }
    }
    out[[i]] <- tibble::tibble(
      gene = rowData(sce)$Gene,
      gene_mean = rowData(sce)$BaseGeneMean,
      # FIXME: Kept for backwards compatibility.
      fc = mean_scores,
      mean_score = mean_scores,
      median_score = median_scores,
      max_score = max_scores,
      direction = directions,
      de_value = de_values
    )
  }

  names(out) <- paste0("group_", seq_len(n_groups))
  out
}

sim <- readRDS(args$sim)
mgs <- find_marker_genes(sim)

saveRDS(mgs, args$mgs)
