library(ggplot2)
library(patchwork)
library(dplyr)

source(here::here("code", "plot-utils.R"))

label_pars <- c(
  "scanpy_wilcoxontiecorrect_rankby_abs",
  "scanpy_t_rankby_abs",
  "edger_ql",
  "scran_t_any",
  "nsforest",
  "scran_wilcox_some",
  "cepo",
  "rankcorr_2",
  "seurat_negbinom",
  "limma_trend",
  "glmgampoi",
  "seurat_t",
  "seurat_MAST",
  "scanpy_wilcoxon_rankby_abs",
  "presto",
  "smash"
)

# Conceptual figure (components)

rank_rank <- readRDS(
  here::here("figures", "raw", "rank-rank-scanpy-default-seurat-default.rds")
)

prop_intersect <- readRDS(
  here::here("figures", "raw", "scanpy-seurat-intersection.rds")
)

conceptual_figure_bottom <- prop_intersect + rank_rank +
  plot_layout(nrow = 1, widths = c(1, 1))

conceptual_figure_bottom

ggsave(
  filename = here::here("figures", "final", "conceptual-figure-bottom.pdf"),
  plot = conceptual_figure_bottom,
  width = 12, height = 4, units = "in"
)

raw_tsne <- readRDS(
  here::here("figures", "raw", "raw-tsne.rds")
)

ggsave(
  filename = here::here("figures", "final", "raw-tsne.pdf"),
  plot = raw_tsne,
  width = 4, height = 4, units = "in"
)

coloured_tsne <- readRDS(
  here::here("figures", "raw", "coloured-tsne.rds")
)

ggsave(
  filename = here::here("figures", "final", "coloured-tsne.pdf"),
  plot = coloured_tsne,
  width = 4, height = 4, units = "in"
)

ms4a1_expression <- readRDS(
  here::here("figures", "raw", "ms4a1-expression.rds")
)

ggsave(
  filename = here::here("figures", "final", "ms4a1-expression.pdf"),
  plot = ms4a1_expression,
  width = 4, height = 4, units = "in"
)

# Method concordance + characteristics figure

concordance_dendorogram <- readRDS(
  here::here("figures", "raw", "dendrogram.rds")
)

raw_info <- readRDS(
  here::here("figures", "raw", "raw-info.rds")
) +
  theme(plot.margin = margin(-1, -1, 1, -1, "cm"))

strategy_info <- readRDS(
  here::here("figures", "raw", "strategy-info.rds")
) +
  theme(plot.margin = margin(-1, -1, -1, -1, "cm"))

package_info <- readRDS(
  here::here("figures", "raw", "package-info.rds")
) +
  theme(plot.margin = margin(-1, -1, -1, -1, "cm"))

general_metrics <- readRDS(
  here::here("figures", "raw", "general-metrics.rds")
) +
  labs(y = "Methods (alphabetical order)") +
  ggtitle("General metrics")

effect_size_metrics <- readRDS(
  here::here("figures", "raw", "effect-size-metrics.rds")
) +
  ggtitle("Effect size metrics")

# overall_prop_up <- readRDS(
#   here::here("figures", "raw", "overall-prop-up.rds")
# ) +
#   ggtitle("Regulation direction across datasets")
#
# all_n_zero_pvalues <- readRDS(
#   here::here("figures", "raw", "all_n_zero_pvalues.rds")
# ) +
#   ggtitle("Number of zero p-values across datasets")

full_dendrogram <- guide_area() + concordance_dendorogram +
  package_info +
  raw_info +
  strategy_info +
  plot_layout(
    nrow = 1,
    widths = c(0.2, 0.9, 0.02, 0.02, 0.02),
    heights = c(1, 4, 1, 1, 1),
    guides = "collect"
  )

full_dendrogram

concordance_figure <- full_dendrogram / (general_metrics + effect_size_metrics) +
  plot_layout(heights = c(4, 4), tag_level = "new") +
  plot_annotation(tag_levels = list(c("a", " ", " ", " ", "b", "c")))  &
  theme(plot.tag = element_text(size = 18))

concordance_figure

ggsave(
  filename = here::here("figures", "final", "concordance-figure.pdf"),
  plot = concordance_figure,
  width = 16,
  height = 18,
  units = "in"
)

# Computational performance + implementation quality figure

overall_time <- readRDS(here::here("figures", "raw", "overall-time.rds"))
overall_memory <- readRDS(here::here("figures", "raw", "overall-memory.rds"))
num_cells_time <- readRDS(here::here("figures", "raw", "time-num-cells.rds")) +
  theme(axis.title.y = element_text(margin = margin(r = -120, unit = "pt")))
implementation_quality <- readRDS(here::here("figures", "raw", "implementation.rds"))

# Warning is spurious
top <- overall_time + overall_memory
bottom <- num_cells_time + implementation_quality +
  plot_layout(widths = c(2, 1))

computational_performance_figure <- top / bottom +
  plot_annotation(tag_levels = "a") &
  theme(plot.tag = element_text(size = 18))

computational_performance_figure

ggsave(
  filename = here::here("figures", "final", "computational-performance-figure.pdf"),
  plot = computational_performance_figure,
  width = 16,
  height = 16,
  units = "in"
)

# Simulated performance figure

overall_recall <- readRDS(
  here::here("figures", "raw", "overall-recall.rds")
) +
  ggtitle("Recall")

overall_precision <- readRDS(
  here::here("figures", "raw", "overall-precision.rds")
) +
  ggtitle("Precision") +
  theme(axis.title.y = element_blank())

overall_f1_score <- readRDS(
  here::here("figures", "raw", "overall-f1-score.rds")
) +
  ggtitle("F1 score") +
  theme(axis.title.y = element_blank())

simulated_perf_figure <- overall_recall + overall_precision + overall_f1_score +
  plot_layout(guides = "collect") +
  plot_annotation(tag_levels = "a") &
  theme(plot.tag = element_text(size = 18))

simulated_perf_figure

ggsave(
  filename = here::here("figures", "final", "simulated-perf-figure.pdf"),
  plot = simulated_perf_figure,
  width = 14,
  height = 8,
  units = "in"
)

# Expert performance figure

expert_lawlor_heatmap <- readRDS(
  here::here("figures", "raw", "expert-lawlor-heatmap.rds")
)

expert_pbmc3k_heatmap <- readRDS(
  here::here("figures", "raw", "expert-pbmc3k-heatmap.rds")
) +
  theme(axis.title.y = element_blank())

expert_lawlor_n_annotated <- readRDS(
  here::here("figures", "raw", "expert-lawlor-n-annotated.rds")
)

expert_pbmc3k_n_annotated <- readRDS(
  here::here("figures", "raw", "expert-pbmc3k-n-annotated.rds")
) +
  theme(axis.title.y = element_blank())

expert_perf_figure <- expert_lawlor_heatmap + expert_pbmc3k_heatmap +
  expert_lawlor_n_annotated + expert_pbmc3k_n_annotated +
  plot_layout(ncol = 2, guides = "collect") +
  plot_annotation(tag_levels = "a") &
  theme(plot.tag = element_text(size = 18))

expert_perf_figure

ggsave(
  filename = here::here("figures", "final", "expert-perf-figure.pdf"),
  plot = expert_perf_figure,
  width = 16,
  height = 16,
  units = "in"
)

# Predictive performance figure

zhao_pred_perf <- readRDS(
  here::here("figures", "raw", "zhao-pred-perf.rds")
)

overall_multiclass_pred_plot <- readRDS(
  here::here("figures", "raw", "overall-mc-pred-plot-z-score-knn.rds")
)

pbmc3k_seurat_wilcox_confmat <- readRDS(
  here::here("figures", "raw", "pbmc3k-seurat-wilcox-confmat.rds")
)

lhs <- (pbmc3k_seurat_wilcox_confmat / zhao_pred_perf) +
  plot_layout(ncol = 1, heights = c(1, 2))
pred_perf_figure <- lhs - overall_multiclass_pred_plot +
  plot_layout(ncol = 2) +
  plot_annotation(tag_levels = "a") &
  theme(plot.tag = element_text(size = 18))

pred_perf_figure

ggsave(
  filename = here::here("figures", "final", "pred-perf-figure.pdf"),
  plot = pred_perf_figure,
  width = 14,
  height = 14,
  units = "in"
)

# Case studies figure

log_fc_plot <- readRDS(
  here::here("figures", "raw", "log-fc-b-cluster-scatter.rds")
)
welch_issue_plot <- readRDS(
  here::here("figures", "raw", "rr-scanpy-t-abs-seurat-t.rds")
)
logfc_abs_prop_intersect_plot <- readRDS(
  here::here("figures", "raw", "logfc-abs-prop-intersect.rds")
)
scanpy_seurat_zeisel_oligodendrocytes_plot <- readRDS(
  here::here("figures", "raw", "scanpy-seurat-zeisel-oligodendrocytes.rds")
)

case_studies_figure <- scanpy_seurat_zeisel_oligodendrocytes_plot +
  logfc_abs_prop_intersect_plot +
  welch_issue_plot +
  log_fc_plot +
  plot_layout(ncol = 2) +
  plot_annotation(tag_levels = "a") &
  theme(plot.tag = element_text(size = 18))

case_studies_figure

ggsave(
  filename = here::here("figures", "final", "case-studies-figure.pdf"),
  plot = case_studies_figure,
  width = 14,
  height = 14,
  units = "in"
)

# Supplementary figures

# Additional expert heatmaps

expert_ss3_pbmc3_heatmap <- readRDS(
  here::here("figures", "raw", "expert-ss3_pbmc-heatmap.rds")
) +
  theme(legend.position = "none")
expert_zeisel_heatmap <- readRDS(
  here::here("figures", "raw", "expert-zeisel-heatmap.rds")
)

ss3_pbmc_zeisel_expert_heatmaps <- expert_ss3_pbmc3_heatmap + expert_zeisel_heatmap  +
  plot_annotation(tag_levels = "a") &
  theme(plot.tag = element_text(size = 18))

ggsave(
  here::here("figures", "final", "ss3_pbmc-zeisel-expert-heatmaps.pdf"),
  ss3_pbmc_zeisel_expert_heatmaps,
  width = 16,
  height = 12,
  units = "in"
)

# Additional expert number of cluster annotated plots

expert_ss3_pbmc_n_annotated <- readRDS(
  here::here("figures", "raw", "expert-ss3_pbmc-n-annotated.rds")
) +
  theme(legend.position = "none")

expert_zeisel_n_annotated <- readRDS(
  here::here("figures", "raw", "expert-zeisel-n-annotated.rds")
)

ss3_pbmc_zeisel_expert_n_annotated <- expert_ss3_pbmc_n_annotated +
  expert_zeisel_n_annotated +
  plot_annotation(tag_levels = "a") &
  theme(plot.tag = element_text(size = 18))

ss3_pbmc_zeisel_expert_n_annotated

ggsave(
  here::here("figures", "final", "ss3_pbmc-zeisel-expert-n-annotated.pdf"),
  ss3_pbmc_zeisel_expert_n_annotated,
  width = 16,
  height = 12,
  units = "in"
)

# Number of clusters simulations

time_num_clusters <- readRDS(here::here("figures", "raw", "time-num-clusters.rds"))
memory_num_clusters <- readRDS(here::here("figures", "raw", "memory-num-clusters.rds"))

num_cluster_sims <- time_num_clusters + memory_num_clusters +
  plot_annotation(tag_levels = "a") &
  theme(plot.tag = element_text(size = 18))

ggsave(
  here::here("figures", "final", "num-clusters-sims.pdf"),
  num_cluster_sims,
  width = 20,
  height = 12,
  units = "in"
)

# Zero expression consequences

all_logfc_scatter <- readRDS(
  here::here("figures", "raw", "log-fc-scatter.rds")
) +
  ggtitle("pbmc3k data")
volcano_plot <- readRDS(
  here::here("figures", "raw", "volcano-plot.rds")
)
zero_expression <- all_logfc_scatter + volcano_plot +
  plot_annotation(tag_levels = "a") &
  theme(plot.tag = element_text(size = 18))
zero_expression

ggsave(
  here::here("figures", "final", "zero-expression.pdf"),
  zero_expression,
  width = 12,
  height = 8,
  units = "in"
)

# SVM

svm_overall_pred_perf <- readRDS(
  here::here("figures", "raw", "overall-mc-pred-plot-z-score-svm.rds")
)

svm_zhao <- readRDS(
  here::here("figures", "raw", "zhao-pred-perf-svm.rds")
)

svm_pred_perf <- svm_zhao + svm_overall_pred_perf +
  plot_layout(guides = "collect") +
  plot_annotation(tag_levels = "a") &
  theme(plot.tag = element_text(size = 18))

svm_pred_perf

ggsave(
  here::here("figures", "final", "svm-pred-perf.pdf"),
  svm_pred_perf,
  width = 14,
  height = 12,
  units = "in"
)

# Sum max classifier

sum_max_overall_pred_perf <- readRDS(
  here::here("figures", "raw", "overall-mc-pred-plot-z-score-sum_max.rds")
)

sum_max_zhao <- readRDS(
  here::here("figures", "raw", "zhao-pred-perf-sum_max.rds")
)

sum_max_pred_perf <- sum_max_zhao + sum_max_overall_pred_perf +
  plot_layout(guides = "collect") +
  plot_annotation(tag_levels = "a") &
  theme(plot.tag = element_text(size = 18))

sum_max_pred_perf

ggsave(
  here::here("figures", "final", "sum_max-pred-perf.pdf"),
  sum_max_pred_perf,
  width = 14,
  height = 12,
  units = "in"
)

# Poster figures

poster_case_studies_figure <- scanpy_seurat_zeisel_oligodendrocytes_plot +
  log_fc_plot +
  plot_layout(ncol = 2) +
  plot_annotation(tag_levels = "a") &
  theme(plot.tag = element_text(size = 18)) +
  theme(axis.title = element_text(size = 22))

poster_case_studies_figure

ggsave(
  filename = here::here("figures", "final", "poster-case-studies-figure.pdf"),
  plot = poster_case_studies_figure,
  width = 14,
  height = 8,
  units = "in"
)

poster_pred_perf_figure <- lhs - overall_multiclass_pred_plot +
  plot_layout(ncol = 2) +
  plot_annotation(tag_levels = "a") &
  theme(plot.tag = element_text(size = 18)) +
  theme(
    axis.title = element_text(size = 18),
  )

ggsave(
  filename = here::here("figures", "final", "poster-pred-perf-figure.pdf"),
  plot = poster_pred_perf_figure,
  width = 14,
  height = 14,
  units = "in"
)

poster_simulated_perf_figure <- overall_recall + overall_precision + overall_f1_score +
  plot_layout(guides = "collect") +
  plot_annotation(tag_levels = "a") &
  theme(plot.tag = element_text(size = 18)) +
  theme(
    axis.title = element_text(size = 18),
  )

ggsave(
  filename = here::here("figures", "final", "poster-simulated-perf-figure.pdf"),
  plot = poster_simulated_perf_figure,
  width = 14,
  height = 8,
  units = "in"
)

# Blood datasets expert marker gene figures

yoshida_heatmap <- readRDS(here::here("figures", "raw", "expert-yoshida-heatmap.rds"))
ren_heatmap <- readRDS(here::here("figures", "raw", "expert-ren-heatmap.rds"))
stephenson_heatmap <- readRDS(here::here("figures", "raw", "expert-stephenson-heatmap.rds"))
dominguez_conde_heatmap <- readRDS(here::here("figures", "raw", "expert-dominguez_conde-heatmap.rds"))

blood_heatmaps <- yoshida_heatmap + ren_heatmap + stephenson_heatmap + dominguez_conde_heatmap +
  plot_layout(guides = "collect") +
  plot_annotation(tag_levels = "a") &
  theme(plot.tag = element_text(size = 18))

ggsave(
  filename = here::here("figures", "final", "blood-expert-heatmap.pdf"),
  plot = blood_heatmaps,
  width = 14,
  height = 18,
  units = "in"
)

stephenson_n_annotated <- readRDS(here::here("figures", "raw", "expert-stephenson-n-annotated.rds"))
ren_n_annotated <- readRDS(here::here("figures", "raw", "expert-ren-n-annotated.rds"))
yoshida_n_annotated <- readRDS(here::here("figures", "raw", "expert-yoshida-n-annotated.rds"))
dominguez_conde_n_annotated <- readRDS(here::here("figures", "raw", "expert-dominguez_conde-n-annotated.rds"))

blood_n_annotated <- yoshida_n_annotated + ren_n_annotated +
  stephenson_n_annotated + dominguez_conde_n_annotated +
  plot_layout(guides = "collect") +
  plot_annotation(tag_levels = "a") &
  theme(plot.tag = element_text(size = 18))

ggsave(
  filename = here::here("figures", "final", "blood-expert-n-annotated.pdf"),
  plot = blood_n_annotated,
  width = 14,
  height = 18,
  units = "in"
)

