# This scripts downloads, prepares and saves the 'astrocyte' data.

args <- R.utils::commandArgs(
  trailingOnly = TRUE,
  asValues = TRUE
)

suppressMessages({
  library(SingleCellExperiment)
  library(scran)
  library(scater)
})

set.seed(07112021)

astrocyte <- readRDS(args$raw_data)

colLabels(astrocyte) <- as.factor(colLabels(astrocyte))

clusters <- quickCluster(astrocyte)
astrocyte <- computeSumFactors(astrocyte, clusters = clusters)
astrocyte <- logNormCounts(astrocyte)

dec_astrocyte <- modelGeneVarByPoisson(astrocyte)
top_astrocyte <- getTopHVGs(dec_astrocyte, n = 2000)
astrocyte <- astrocyte[top_astrocyte, ]

astrocyte <- runPCA(astrocyte)
astrocyte <- runTSNE(astrocyte, dimred = "PCA")
astrocyte <- runUMAP(astrocyte, dimred = "PCA")

saveRDS(astrocyte, args$data)
