args <- R.utils::commandArgs(
  trailingOnly = TRUE,
  asValues = TRUE
)

suppressMessages({
  library(scran)
  library(scater)
})

set.seed(406023)

dominguez_conde <- readRDS(args$raw_data)

small_clusters <- names(which(table(dominguez_conde$Curated_annotation) < 200))
dominguez_conde <- dominguez_conde[, !(dominguez_conde$Curated_annotation %in% small_clusters)]

clusters <- quickCluster(dominguez_conde)
dominguez_conde <- computeSumFactors(dominguez_conde, clusters = clusters)
dominguez_conde <- logNormCounts(dominguez_conde)

blood_expert_mgs_info <- readRDS(
  here::here("data", "expert_mgs", "blood_expert_mgs.rds")
)
dominguez_conde_ind <- blood_expert_mgs_info$cluster %in% dominguez_conde$Curated_annotation
dominguez_conde_expert_mgs_info <- blood_expert_mgs_info[dominguez_conde_ind, ]
dominguez_conde_expert_mgs <- unlist(dominguez_conde_expert_mgs_info$expert_mgs)

dec_dominguez_conde <- modelGeneVarByPoisson(dominguez_conde)
top_dominguez_conde <- getTopHVGs(dec_dominguez_conde, n = 2000)
dominguez_conde <- dominguez_conde[union(top_dominguez_conde, dominguez_conde_expert_mgs), ]

dominguez_conde <- runPCA(dominguez_conde)
dominguez_conde <- runTSNE(dominguez_conde, dimred = "PCA")
dominguez_conde <- runUMAP(dominguez_conde, dimred = "PCA")

colLabels(dominguez_conde) <- factor(dominguez_conde$Curated_annotation)

saveRDS(dominguez_conde, args$data)
