# This scripts downloads, prepares and saves the `lawlor` data.

args <- R.utils::commandArgs(
  trailingOnly = TRUE,
  asValues = TRUE
)

suppressMessages({
  library(scRNAseq)
  library(AnnotationHub)
  library(AnnotationDbi)
  library(EnsDb.Hsapiens.v86)
  library(scater)
  library(scran)
})

set.seed(24072021)

lawlor <- readRDS(args$raw_data)

anno <- select(
  EnsDb.Hsapiens.v86,
  keys = rownames(lawlor),
  keytype = "GENEID",
  columns = c("SYMBOL", "SEQNAME")
)
rowData(lawlor) <- anno[match(rownames(lawlor), anno[, 1]), -1]

old_rownames <- rownames(lawlor)
rownames(lawlor) <- rowData(lawlor)$SYMBOL
is_na_rownames <- which(is.na(rownames(lawlor)))
rownames(lawlor)[is_na_rownames] <- old_rownames[is_na_rownames]

stats <- perCellQCMetrics(
  lawlor,
  subsets = list(Mito = which(rowData(lawlor)$SEQNAME == "MT"))
)

qc <- quickPerCellQC(
  stats,
  percent_subsets = "subsets_Mito_percent",
  batch = lawlor$`islet unos id`
)

lawlor <- lawlor[, !qc$discard]

clusters <- quickCluster(lawlor)
lawlor <- computeSumFactors(lawlor, clusters = clusters)
lawlor <- logNormCounts(lawlor)

lawlor_expert_mgs_info <- readRDS(
  here::here("data", "expert_mgs", "lawlor_expert_mgs.rds")
)
lawlor_expert_mgs <- unlist(lawlor_expert_mgs_info$expert_mgs)

dec_lawlor <- modelGeneVarByPoisson(lawlor)
top_lawlor <- getTopHVGs(dec_lawlor, n = 2000)
lawlor <- lawlor[union(top_lawlor, lawlor_expert_mgs), ]

lawlor <- runPCA(lawlor)
lawlor <- runTSNE(lawlor, dimred = "PCA")
lawlor <- runUMAP(lawlor, dimred = "PCA")

colData(lawlor)$`cell type` <- factor(colData(lawlor)$`cell type`)
colLabels(lawlor) <- colData(lawlor)$`cell type`

# Create subsets of the data.
lawlor_alpha <- lawlor[, lawlor$`cell type` == "Alpha"]
lawlor_beta <- lawlor[, lawlor$`cell type` == "Beta"]

# Save all files.
saveRDS(lawlor, args$data)

#saveRDS(lawlor_alpha , here::here("data", "real_data", "lawlor-alpha.rds"))
#saveRDS(lawlor_beta, here::here("data", "real_data", "lawlor-beta.rds"))
