# This scripts prepares and saves the `mesenchymal` data.

args <- R.utils::commandArgs(
  trailingOnly = TRUE,
  asValues = TRUE
)

suppressMessages({
  library(scater)
  library(scran)
  library(Seurat)
})

set.seed(31012022)

mesenchymal <- readRDS(args$raw_data)

# Remove unneeded assays which prevent conversion to a SCE object.
mesenchymal@assays$HTO <- NULL
mesenchymal@assays$SCT <- NULL

mesenchymal <- as.SingleCellExperiment(mesenchymal)

rowData(mesenchymal) <- DataFrame(gene = rownames(mesenchymal))

colData(mesenchymal)$predicted.id <- factor(colData(mesenchymal)$predicted.id)
colLabels(mesenchymal) <- colData(mesenchymal)$predicted.id

clusters <- quickCluster(mesenchymal)
mesenchymal <- computeSumFactors(mesenchymal, clusters = clusters)
mesenchymal <- logNormCounts(mesenchymal)

dec_mesenchymal <- modelGeneVarByPoisson(mesenchymal)
top_mesenchymal <- getTopHVGs(dec_mesenchymal, n = 2000)
mesenchymal <- mesenchymal[top_mesenchymal, ]

saveRDS(mesenchymal, args$data)
