# This script prepares and saves the `paul` data.

args <- R.utils::commandArgs(
  trailingOnly = TRUE,
  asValues = TRUE
)

suppressMessages({
  library(scRNAseq)
  library(scran)
  library(scater)
})

set.seed("17212021")

paul <- readRDS(args$raw_data)

# Clean up the SCE object.
assay(paul, "counts") <- assay(paul, "X")
assay(paul, "X") <- NULL
metadata(paul) <- list()
colnames(paul) <- paste0("Cell", as.numeric(colnames(paul)) + 1)
# Some of the Scanpy manipulations require non-null rowData.
rowData(paul) <- DataFrame(gene = rownames(paul))

clusters <- quickCluster(paul)
paul <- computeSumFactors(paul, clusters = clusters)
paul <- logNormCounts(paul)

dec_paul <- modelGeneVarByPoisson(paul)
top_paul <- getTopHVGs(dec_paul, n = 2000)
paul <- paul[top_paul, ]

paul <- runPCA(paul)
paul <- runTSNE(paul, dimred = "PCA")
paul <- runUMAP(paul, dimred = "PCA")

colLabels(paul) <- paul$paul15_clusters

saveRDS(paul, args$data)
