args <- R.utils::commandArgs(
  trailingOnly = TRUE,
  asValues = TRUE
)

suppressMessages({
  library(scran)
  library(scater)
})

set.seed(406023)

ren <- readRDS(args$raw_data)

ren <- ren[, sample(1:ncol(ren), 25000)]

small_clusters <- names(which(table(ren$Curated_annotation) < 200))
ren <- ren[, !(ren$Curated_annotation %in% small_clusters)]

clusters <- quickCluster(ren)
ren <- computeSumFactors(ren, clusters = clusters)
ren <- logNormCounts(ren)

blood_expert_mgs_info <- readRDS(
  here::here("data", "expert_mgs", "blood_expert_mgs.rds")
)
ren_ind <- blood_expert_mgs_info$cluster %in% ren$Curated_annotation
ren_expert_mgs_info <- blood_expert_mgs_info[ren_ind, ]
ren_expert_mgs <- unlist(ren_expert_mgs_info$expert_mgs)

dec_ren <- modelGeneVarByPoisson(ren)
top_ren <- getTopHVGs(dec_ren, n = 2000)
ren <- ren[union(top_ren, ren_expert_mgs), ]

ren <- runPCA(ren)
ren <- runTSNE(ren, dimred = "PCA")
ren <- runUMAP(ren, dimred = "PCA")

colLabels(ren) <- factor(ren$Curated_annotation)

saveRDS(ren, args$data)
