args <- R.utils::commandArgs(
  trailingOnly = TRUE,
  asValues = TRUE
)

suppressMessages({
  library(scran)
  library(scater)
})

set.seed(406023)

stephenson <- readRDS(args$raw_data)

stephenson <- stephenson[, sample(1:ncol(stephenson), 25000)]

small_clusters <- names(which(table(stephenson$Curated_annotation) < 200))
stephenson <- stephenson[, !(stephenson$Curated_annotation %in% small_clusters)]

clusters <- quickCluster(stephenson)
stephenson <- computeSumFactors(stephenson, clusters = clusters)
stephenson <- logNormCounts(stephenson)

blood_expert_mgs_info <- readRDS(
  here::here("data", "expert_mgs", "blood_expert_mgs.rds")
)
stephenson_ind <- blood_expert_mgs_info$cluster %in% stephenson$Curated_annotation
stephenson_expert_mgs_info <- blood_expert_mgs_info[stephenson_ind, ]
stephenson_expert_mgs <- unlist(stephenson_expert_mgs_info$expert_mgs)

dec_stephenson <- modelGeneVarByPoisson(stephenson)
top_stephenson <- getTopHVGs(dec_stephenson, n = 2000)
stephenson <- stephenson[union(top_stephenson, stephenson_expert_mgs), ]

stephenson <- runPCA(stephenson)
stephenson <- runTSNE(stephenson, dimred = "PCA")
stephenson <- runUMAP(stephenson, dimred = "PCA")

colLabels(stephenson) <- factor(stephenson$Curated_annotation)

saveRDS(stephenson, args$data)
