args <- R.utils::commandArgs(
  trailingOnly = TRUE,
  asValues = TRUE
)

suppressMessages({
  library(scran)
  library(scater)
})

set.seed(406023)

yoshida <- readRDS(args$raw_data)

small_clusters <- names(which(table(yoshida$Curated_annotation) < 200))
yoshida <- yoshida[, !(yoshida$Curated_annotation %in% small_clusters)]

clusters <- quickCluster(yoshida)
yoshida <- computeSumFactors(yoshida, clusters = clusters)
yoshida <- logNormCounts(yoshida)

blood_expert_mgs_info <- readRDS(
  here::here("data", "expert_mgs", "blood_expert_mgs.rds")
)
yoshida_ind <- blood_expert_mgs_info$cluster %in% yoshida$Curated_annotation
yoshida_expert_mgs_info <- blood_expert_mgs_info[yoshida_ind, ]
yoshida_expert_mgs <- unlist(yoshida_expert_mgs_info$expert_mgs)

dec_yoshida <- modelGeneVarByPoisson(yoshida)
top_yoshida <- getTopHVGs(dec_yoshida, n = 2000)
yoshida <- yoshida[union(top_yoshida, yoshida_expert_mgs), ]

yoshida <- runPCA(yoshida)
yoshida <- runTSNE(yoshida, dimred = "PCA")
yoshida <- runUMAP(yoshida, dimred = "PCA")

colLabels(yoshida) <- factor(yoshida$Curated_annotation)

saveRDS(yoshida, args$data)
