# This scripts downloads, prepares and saves the `zeisel` data.

args <- R.utils::commandArgs(
  trailingOnly = TRUE,
  asValues = TRUE
)

suppressMessages({
  library(scRNAseq)
  library(scran)
  library(scater)
})

set.seed("24072021")

zeisel <- readRDS(args$raw_data)

clusters <- quickCluster(zeisel)
zeisel <- computeSumFactors(zeisel, clusters = clusters)
zeisel <- logNormCounts(zeisel)

zeisel_expert_mgs_info <- readRDS(
  here::here("data", "expert_mgs", "zeisel_expert_mgs.rds")
)
zeisel_expert_mgs <- unlist(zeisel_expert_mgs_info$expert_mgs)

dec_zeisel <- modelGeneVarByPoisson(zeisel)
top_zeisel <- getTopHVGs(dec_zeisel, n = 2000)
zeisel <- zeisel[union(top_zeisel, zeisel_expert_mgs), ]

zeisel <- runPCA(zeisel)
zeisel <- runTSNE(zeisel, dimred = "PCA")
zeisel <- runUMAP(zeisel, dimred = "PCA")

zeisel$level1class <- factor(zeisel$level1class)
colLabels(zeisel) <- zeisel$level1class

# We do not use them and they break conversion to a Seurat object.
zeisel <- removeAltExps(zeisel)

saveRDS(zeisel, args$data)
