# This scripts prepares and saves the `zhao` data.

args <- R.utils::commandArgs(
  trailingOnly = TRUE,
  asValues = TRUE
)

suppressMessages({
  library(scRNAseq)
  library(scater)
  library(scran)
})

set.seed(07122021)

zhao <- readRDS(args$raw_data)

# Remove cells without annotations.
zhao <- zhao[, !is.na(zhao$fine)]

rownames(zhao) <- rowData(zhao)$Symbol

clusters <- quickCluster(zhao)
zhao <- computeSumFactors(zhao, clusters = clusters)
zhao <- logNormCounts(zhao)

dec_zhao <- modelGeneVarByPoisson(zhao)
top_zhao <- getTopHVGs(dec_zhao, n = 2000)
zhao <- zhao[top_zhao, ]

zhao <- runPCA(zhao)
zhao <- runTSNE(zhao, dimred = "PCA")
zhao <- runUMAP(zhao, dimred = "PCA")

colLabels(zhao) <- factor(zhao$fine)

saveRDS(zhao, args$data)
