
# Source needed utility functions.
source(here::here("code", "seurat-utils.R"))

#' Find marker genes using cosgr
#'
#' @param sce A SingleCellExperiment object,
#' @param pars A list of parameters passed to the function.
#'
#' @return A list containing the time of the computation and results of the
#'  method.
#'
run_cosg <- function(sce, pars) {

  seurat_data <- as_seurat_data(sce)

  times <- system.time({
    raw_result <- COSG::cosg(
      seurat_data,
      groups = "all",
      assay = "RNA",
      slot = "data",
      mu = 1,
      n_genes_user = 200
    )
  })

  result <- dplyr::bind_cols(
    tidyr::pivot_longer(
      raw_result$names,
      cols = everything(),
      names_to = "cluster",
      values_to = "gene"
    ),
    dplyr::select(
      tidyr::pivot_longer(
        raw_result$scores,
        cols = everything(),
        names_to = "cluster",
        values_to = "raw_statistic"
      ),
      -cluster)
  )

  # See note in `run-nsForest.R`
  result <- dplyr::filter(result, gene %in% rownames(sce))

  lfcs <- calculate_log_fc(sce, genes = result$gene, clusters = result$cluster)

  result <- dplyr::mutate(
    result,
    log_fc = lfcs,
    p_value = 0,
    p_value_adj = 0,
    scaled_statistic = 0
  )
  result <- tibble::as_tibble(result)

  # Get the elapsed time.
  time <- times[[3]]

  validate_mgs_result(result)

  list(time = time, result = result, raw_result = raw_result, pars = pars)
}
