#' Find marker genes using glmGamPoi.
#'
#' Wraps functions from the glmGamPoi package.
#'
#' @param sce A SingleCellExperiment object
#' @param pars parameters passed to the function
#'
#' @return A list containing the time of the computation and results of the
#'  method.
#'
run_glmgampoi <- function(sce, pars) {

  times <- system.time({

    n_groups <- length(unique(colLabels(sce)))
    out <- vector("list", length = n_groups)

    for (j in seq_len(n_groups)) {

      group <- levels(colLabels(sce))[[j]]
      x <- factor(as.numeric(colLabels(sce) == group))

      count_mat <- as.matrix(counts(sce))
      glm_gp_fit <- glmGamPoi::glm_gp(
        count_mat,
        design = ~ 1 + x,
        on_disk = FALSE
      )
      # For some reason the x varaible above is renamed to x1.
      de_res <- glmGamPoi::test_de(
        glm_gp_fit,
        contrast = "x1",
        sort_by = "pval"
      )

      de_res <- dplyr::rename(
        de_res,
        gene = name,
        p_value = pval,
        p_value_adj = adj_pval,
        scaled_statistic = f_statistic,
        log_fc = lfc
      )

      de_res <- dplyr::select(de_res, -c(df1, df2))

      de_res <- dplyr::mutate(
        de_res,
        raw_statistic = 0,
        cluster = group
      )

      de_res <- tibble::as_tibble(de_res)

      out[[j]] <- de_res
    }

    result <- dplyr::bind_rows(!!!out)
  })

  # Get the elapsed time.
  time <- times[[3]]

  validate_mgs_result(result)

  list(time = time, result = result, raw_result = result, pars = pars)
}
