
suppressMessages({
  library(R.utils)
  library(jsonlite)
  library(SingleCellExperiment)
  library(sessioninfo)
})

# Get the passed arguments from the command line via Snakemake.
args <- commandArgs(trailingOnly = TRUE, asValues = TRUE)

# Debugging
print(args)

pbmc3k <- readRDS(here::here("data", "real_data", "pbmc3k.rds"))
pbmc3k <- pbmc3k[, pbmc3k$label != "Platelet"]
colLabels(pbmc3k) <- factor(colLabels(pbmc3k),
                            levels = c("Naive CD4 T", "CD14+ Mono",
                                       "Memory CD4 T", "B", "CD8 T", "FCGR3A+ Mono",
                                       "NK", "DC"))

downsample_cells <- function(sce, percent) {
  if (percent == 100) {
    return(sce)
  }
  n <- ncol(sce)
  is_good_sample <- function(sce) {
    length(unique(colLabels(sce))) == 8 && all(table(colLabels(sce)) > 10)
  }
  sample_ind <- sample(1:n, size = n * (percent / 100), replace = FALSE)
  out <- sce[, sample_ind]

  good_sample <- is_good_sample(out)
  while (!good_sample) {
    sample_ind <- sample(1:n, size = n * (percent / 100), replace = FALSE)
    out <- sce[, sample_ind]
    good_sample <- is_good_sample(out)
  }
  out
}


percent_file <- args$percent
meth_pars <- as.list(fromJSON(args$meth_pars))
method_file <- args$fun
result_file <- args$result

percent <- readRDS(percent_file)
data <- downsample_cells(pbmc3k, percent)

source(here::here("code", "run-utils.R"))
source(method_file)
func <- gsub("(.R)", "", basename(method_file))
func <- gsub("-", "_", func)
result <- get(func)(data, meth_pars)

# Save the results.
saveRDS(result, result_file)

session_info()
