
suppressMessages({
  library(R.utils)
  library(jsonlite)
  library(SingleCellExperiment)
  library(sessioninfo)
})

# Get the passed arguments from the command line via Snakemake.
args <- commandArgs(trailingOnly = TRUE, asValues = TRUE)

# Debugging
print(args)

pbmc3k <- readRDS(here::here("data", "real_data", "pbmc3k.rds"))

pbmc3k_cluster_order <- c("Naive CD4 T", "CD14+ Mono", "Memory CD4 T", 
                          "Platelet", "CD8 T", "FCGR3A+ Mono", "NK", "DC", "B")

remove_clusters <- function(sce, n, cluster_order) {
  if (n == 0) {
    return(sce)
  } 
  
  clusters_to_remove <- cluster_order[1:n] 
  remove_ind <- colLabels(sce) %in% clusters_to_remove
  n_clusters <- length(cluster_order)
  colLabels(sce) <- factor(colLabels(sce), 
                           levels = cluster_order[(n + 1):n_clusters])
  sce[, !remove_ind] 
}

n_file <- args$n
meth_pars <- as.list(fromJSON(args$meth_pars))
method_file <- args$fun
result_file <- args$result

n <- readRDS(n_file)
data <- remove_clusters(pbmc3k, n, pbmc3k_cluster_order)

source(here::here("code", "run-utils.R"))
source(method_file)
func <- gsub("(.R)", "", basename(method_file))
func <- gsub("-", "_", func)
result <- get(func)(data, meth_pars)

# Save the results.
saveRDS(result, result_file)

session_info()
