#' Find marker genes using limma
#'
#' @param sce A SingleCellExperiment object.
#' @param pars parameters passed to the function.
#'
#' @return A list containing the time of the computation and results of the
#'  method.
#'
run_random <- function(sce, pars) {

  times <- system.time({

    result <- tidyr::expand_grid(
      gene = rownames(sce),
      cluster = levels(colLabels(sce))
    )
    n <- nrow(result)

    # These values/distributions were chosen haphazardly.
    result$raw_statistic <- rnorm(n, 0, 2)
    result$p_value <- runif(n, 0, 1)
    result$p_value_adj <- result$p_value / 10
    result$scaled_statistic <- rnorm(n, 0, 2)
    result$log_fc <- rnorm(n, 0, 3)
  })

  # Use elapsed time.
  time <- times[[3]]

  list(result = result, raw_result = result, time = time, pars = pars)
}
