#' Find marker genes using scanpy
#'
#' @param sce A SingleCellExperiment object,
#' @param pars A list of parameters passed to the function.
#'
#' @return A list containing the time of the computation and results of the
#'  method.
#'
run_scgenefit <- function(sce, pars) {

  library(reticulate)

  scgf_funcs <- import("scGeneFit.functions")
  np <- import("numpy")

  X <- matrix(logcounts(sce), nrow = ncol(sce), ncol = nrow(sce))

  lookup <- 1:length(unique(colLabels(sce)))
  names(lookup) <- levels(colLabels(Group))

  factor_labels <- as.character(colLabels(sce))
  numeric_labels <- as.integer(unname(lookup[factor_labels]))

  times <- system.time({
    markers <- scgf_funcs$get_markers(
      data = X,
      labels = numeric_labels,
      num_markers = as.integer(25),
      method = "centres",
      redundancy = 0.25
    )
  })

  genes <- rownames(sce)[markers]

  # We use the 'elapsed' time.
  time <- times[[3]]

  list(time = time, result = genes, pars = pars)
}
